% Function: dynamic_perturbation
% Written by Alessandro Mennuni
% Rewritten in December 2011 after a suggestion by LU, Yang
%
% Format: 
% function [y_sim_local,x_sim_local,y_sim_lev,x_sim_lev,ff,pp,error_local,...
%  error_global] = dynamic_perturbation(fun,fun2,sigm,sigm_true,...
%  x_,nx,nA,ny,x_0,T,t_end,eps_s,sec_ord);
% Purpose: Solves a dynamic system of equations f(x1,y1,x,y)=0 where x is a
% vector of nx state variables arranged so that shocks enter in the 
% lower part of the vector, after the other state variables. y is a vector 
% containing the remaining ny variables, see Appendix B in my paper Labor 
% force composition and aggregate fluctuations.
%
% Inputs: 
% fun,fun2,sigm,sigm_true,x_,nx,nA,ny,x_0,T,t_end,eps_s, sec_ord
% fun contains the equations to be solved f(x1,y1,x,y)
% fun2 contains the equations to be solved, however here they are solved 
% for xx=[x1;y] given x and a policy function gy to compute y1=gy(x1),which 
% linearized around the point (x_,y_), takes the form y1=y_+ff_*(x1-x_). 
% So fun2 is function of (xx,x_,y_,ff_,x). (see step 7 in Appendix B)
% sigm is a matrix nx*nx of variances for the shocks to drow the path from 
% x0 to xss,typically this matrix contains zeros. (step 4 in Appendix B)
% sigm_true is a matrix nx*nx of variances of the shocks, non zero entries
% should enter in the i,j associated to the shocks in the vector x. (The 
% policy approximated around (x1,y1,x,y) takes the form 
% x1_=x1+pp(x_-x)+sigm_true*eps, so sigm_true*eps should have 
% non-zero entries only in the last nA rows).
% x_ contains the steady state values of x.
% nx is the dimension of x.
% nA is the dimension of the shock process (the lower part of x)
% ny is the dimension of the other variables y.
% x_0 is the initial condition for x.
% T is the lenght of the time series drown from x_0, along which to proced
% backward as explained in Appendix B.
% t_end is the lenght of the time series for the shocks.
% eps_s is a vector of dimention nA*T containing shock innovations 
% with variance 1.
% sec_ord is a dummy if ==1 the initial path is drawn through linear
% policies, if >1 the path is drawn through quadratic policies.
%
% Outputs: 
% y_sim_local,x_sim_local the simulated x{0,T} and y{0,T} using 2nd order 
% perturbation around the ss
% y_sim_lev,x_sim_lev the simulatied x{0,T} and y{0,T} using dynamic 
% perturbation along the equilibrium path.
% ff,pp,matrixes containig the parameters of the policies so that 
% x_sim(:,t+1)=pp(:,:,t)* (x_sim(:,t)-x_sim(:,t))+x_sim1(:,t)+...
% sigm_true*eps_s(:,t);
% y_sim(:,t)=ff(:,:,t)* (x_sim(:,t)-x_sim(:,t))+y_sim(:,t);
% error_local: a vector of errors f(Ex1,Ey1,x,y) using 2nd order
% perturbation of the ss.
% error_global: a vector of errors f(Ex1,Ey1,x,y) using dynamic
% perturbation along the equilibrium path.
%
% Calls: grad, solab2, solab_stabil, solab, tracem, compute_error_local.
% solab2, solab, tracem are written by Klein and Gomme and are used to
% derive taylor expansions.

function [y_sim_local,x_sim_local,y_sim_lev,x_sim_lev,ff,pp,error_local,error_global] = dynamic_perturbation(fun,fun2,sigm,sigm_true,x_,nx,nA,ny,x_0,T,t_end,eps_s,sec_ord);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1 Find appropriate policy functions at $x_0=x_ss$.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1.1 Apply the perturbation method around the steady state
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
gam=.001*(nx+ny);%.000001;%param for step in change in x to find new policies
x1=x_(1:nx,1);% state vars tomorrow
y1=x_(nx+1:nx+ny,1);% jump tomoroow
x=x_(nx+ny+1:nx+ny+nx,1);% state vars 
y=x_(2*nx+ny+1:2*(nx+ny),1);%jump today
%evaluate eventual mistake at deterministic ss
x_val=[x1;y1;x;y];
err=max(abs(feval(fun,x_val)))

x_ss=x;%fix ss state var as x will be updated
y_ss=y;% same for jumps

%%find gradient (or Jacobian)
%%find Hessian

%{
[gra]=grad(fun,x1,y1,x,y,0);% 0 rather than 1 if no hessian (1st order)

m = size(gra,1);   % Number of variables
aa = gra(:,1:m);
bb = -gra(:,m+1:end);

[ff_ss,pp_ss] = solab(aa,bb,nx);  % Linear approximation

%}
%%find Hessian
if sec_ord>1
    [gra,hes]=grad(fun,x1,y1,x,y,1);
    m = size(gra,1);   % Number of variables

    %Klein-Gomme algorithm to find ss policy functions
    [ff_ss,pp_ss,ee,gg,kx,ky]=solab2(gra,hes,sigm_true,nx);
else
    [gra]=grad(fun,x1,y1,x,y,0);% 0 rather than 1 if no hessian (1st order)
    m = size(gra,1);   % Number of variables
    aa = gra(:,1:m);
    bb = -gra(:,m+1:end);

    [ff_ss,pp_ss] = solab(aa,bb,nx);  % Linear approximation
    ee=zeros(nx*ny,nx);
    gg=zeros(nx*nx,nx);
    kx=zeros(nx,1);
    ky=zeros(ny,1);
end



% initial path

x_sim=zeros(nx,T);y_sim=zeros(ny,T);

x_sim(:,1)=x_0-x;%initial state vars
%y_sim=zeros(nx,T);

% x_{t+1} = kx + Px_t + (1/2)*kron(eye(nx),x'_t)Gx_t + eps_{t+1}
% y_{t}   = ky + Fx_t + (1/2)*kron(eye(ny),x'_t)Ex_t
 
for t=1:T-1
      x_sim(:,t+1)=kx+pp_ss*x_sim(:,t)+ (1/2)*kron(eye(nx),x_sim(:,t)')*gg*x_sim(:,t) +sigm*eps_s(:,t+1);
      %sigm.^2*kx+pp*xs(:,t)+ (1/2)*kron(eye(nx),xs(:,t)')*gg*xs(:,t) +sigm*eps_s(:,t+1);
      y_sim(:,t)  =ky+ff_ss*x_sim(:,t)+ (1/2)*kron(eye(ny),x_sim(:,t)')*ee*x_sim(:,t);

%      x_sim(:,t+1)=pp_ss* x_sim(:,t)+sigm*eps_s(:,t+1);%no shocks for this simul!
%      y_sim(:,t)=ff_ss*x_sim(:,t);
end
y_sim(:,T)  =ky+ff_ss*x_sim(:,T)+ (1/2)*kron(eye(ny),x_sim(:,T)')*ee*x_sim(:,T);
%y_sim(:,T)=ff_ss*x_sim(:,T);
%y_sim_expected(:,1)=y_sim(:,1);
%x_sim_expected(:,1)=x_sim(:,1);

y_sim=y_sim+y_ss*ones(1,T);
x_sim=x_sim+x_ss*ones(1,T);
y_sim_ss=y_sim;
x_sim_ss=x_sim;
y_sim2=y_sim;
x_sim2=x_sim;

%{
figure
subplot(2,1,1)
plot(x_sim(1:nx-nA,2:T)')
xlabel('time')
ylabel('k prime')
subplot(2,1,2)
plot(x_sim(nx-nA+1:nx,2:T)')
xlabel('time')
ylabel('A prime')

%}

%%%start big loop until time series don't converge
 

tol_x_gap=.0;%condition on dx above which the policy is reapproximated
t_init=1;%until where you go backward
% next loop applies perturbation backward from ss until x(t_init), from
% x(0) till x(47)
while t_init<t_end% more or less when the real time sereis ends
    t_init=t_init+1;
    %Find polcies at x_(t_init)
    t=T+1;

    j=1;% index that measures iteration of changes of x, if j=1 x is x_ss, the max j is s.t. x=x_0.
    pp(:,:,1)=pp_ss;x(:,1)=x_ss;x1(:,1)=x_ss;
    ff(:,:,1)=ff_ss;y(:,1)=y_ss;y1(:,1)=y_ss;
    
  
    
    xtemp(:,1)=x_ss;
    alph=1;
    % next loop goes backward on the equilibrium path from ss until it 
    %reaches x_0. At each point it computes new policies
    while t>t_init% at t=t_init+1 we find x_(t_init)
        t=max(t-1,t_init);
        %{
        %make bigg step when far from x_(t_init)
        if t>=60
            t=max(t-4,t_init);% I go backward by 4 periods unless I got to t=t_init
        elseif t>=t_end && t<60
            t=max(t-3,t_init);% I go backward by 2 periods unless I got to t=t_init
        elseif t>=15 && t<t_end
            t=max(t-2,t_init);% I go backward by 1 periods unless I got to t=t_init 
        elseif t<15
            t=max(t-1,t_init);% I go backward by 1 period unless I got to t=t_init 
        end
        %}      
        xtemp(:,j)=x_sim(:,t);
        x_sim_t_1=x_sim(:,t-1);
        % do the following until xtemp(:,j)==x_sim(:,t-1), then move to
        % t=t-1
        while norm(xtemp(:,j)-x_sim_t_1)>tol_x_gap% otherwise backward step 
            %is too small and it is useless to compute new policy
            j=j+1;
            % alph s.t. backward step <=gam
            alph=min(gam/norm( xtemp(:,j-1)-x_sim(:,t-1)),1);
            %if alph<1
             %   disp('alph<1')% just to know whether we are making a step bigger than gam^.5
                %alph=1;
            %end
            xtemp(:,j)=alph*x_sim(:,t-1)+(1-alph)*xtemp(:,j-1);
            x1(:,j)=pp(:,:,j-1)* (xtemp(:,j)-x(:,j-1))+x1(:,j-1); 
            y(:,j)=ff(:,:,j-1)* (xtemp(:,j)-x(:,j-1))+y(:,j-1);
            %to compute y1(x1) find j s.t. policy approximated at closest
            %point to x1(:,j). That j is called i.
            i=1;
            pen=norm(xtemp(:,i)-x1(:,j));
           % i=2;
            if j>2%if j=2 we are making first move from ss, there is no other policy to choose.
                for v=1:j-1
                    if norm(x(:,v)-x1(:,j))<pen
                        pen=norm(x(:,v)-x1(:,j));
                        i=v;
                    end
                end
                %{
                while  norm(xtemp(:,i)-x1(:,j))<=pen && i<j 
                    pen=norm(xtemp(:,i)-x1(:,j));
                    i=i+1;
                end %we are making first move from ss, there is no other policy to choose.
                i=i-1;%or maybe not?
                %}
            else
                i=1;
            end
            %i
            %i=j-1;
            
            x(:,j)=xtemp(:,j);
            %y1(:,j)=ff(:,:,j-1)* (x1(:,j)-x(:,j-1))+y(:,j-1);
            y1(:,j)=ff(:,:,i)* (x1(:,j)-x(:,i))+y(:,i);%correct one
            %if f=0 perturbate and find g_j(.).
            x_val=[x1(:,j);y1(:,j);x(:,j);y(:,j)];
             %x_val=[x_ss;y_ss;x_ss;y_ss];
            if max(abs(feval(fun,x_val)))>10^-11 %point 3 in the notes.
                %disp('error')
                 %max(abs(feval(fun,x_val)))
               %find x1 y s.t fun=0
               xx_=[x1(:,j);y(:,j)];
               options = optimset('Display','off','TolFun',1e-13);  % Turn off display
             
                % Convert to function handle as needed.
                fun2 = fcnchk(fun2);  
               [xx,fv]=fsolve(@(xx)fun2(xx,x(:,i),y(:,i),ff(:,:,i),x(:,j)),xx_,options);
               %[xx,fv]=fsolve(@(xx)fun2(xx,x,y,ff,x(:,j)),xx_,options);
               
               if     max(abs(fv))>10^-6 | isreal(fv)==0 %fsolve failed
                   if i>1%try using the next closest policy function
                       i=i-1;
                           [xx,fv]=fsolve(@(xx)fun2(xx,x(:,i),y(:,i),ff(:,:,i),x(:,j)),xx_,options);
                   end
               end
               
               if max(abs(fv))<10^-6 && isreal(fv)==1
                   x1(:,j)=xx(1:nx,1);y(:,j)=xx(nx+1:nx+ny,1);              
                   y1(:,j)=ff(:,:,i)* (x1(:,j)-x(:,i))+y(:,i);
               else
                   disp('equations not solved') 
                   pause
               end
               %x_val=[x1(:,j);y1(:,j);x(:,j);y(:,j)];
               %disp('error post')
                % max(abs(feval(fun,x_val)))
               %max(abs(fv))
            end 
            [gra]=grad(fun,x1(:,j),y1(:,j),x(:,j),y(:,j),0);% 0 if no hessian (1st order)
            aa = gra(:,1:m);
            bb = -gra(:,m+1:end);
            %[ff(:,:,j),pp(:,:,j)]=solab(aa,bb,nx);%Linear approximation
            stability=1;
            [ff(:,:,j),pp(:,:,j),stability]=solab_stabil(aa,bb,nx);%Linear approximation

            if stability==2 % in this case solution is indeterminate, so I assume they use the next stable ones.
                pause
                ff(:,:,j)=ff(:,:,j-1);
                pp(:,:,j)=pp(:,:,j-1);
            end
                
             stability=stability;
        end
            
    end
 disp('found polucy at x_t for t=')
 t_init
    %found policy at x_{t-1}!
    %store true policy function parameters at x_{t-1}
    ff_(:,:,t)=ff(:,:,j);pp_(:,:,t)=pp(:,:,j);x1_(:,t)=x1(:,j);
    xt_(:,t)=x(:,j);y_(:,t)=y(:,j);
    %now compute x_{t},y_{t-1}

        x_sim_lev(:,t)=pp(:,:,j)*( x(:,j)-x(:,j))+x1(:,j)+...
            sigm_true*eps_s(:,t);
 
        %pp(:,:,j)*(x_sim_lev(:,j)-x(:,j))+x1(:,j)+sigm_true*eps_s(:,t);
        y_sim_lev(:,t-1) =y(:,j);%y(:,j)=ff(:,:,j)* (x(:,j)-x(:,j))+y(:,j);
        %t=t_init

        %%%%%%%%%%%%%%%%%r%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %compute certainty equivalent error f(x1,y1,x,y)
        %to do this I need to fins y1(x1)
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        y1_(:,t)=y1(:,j);
        x_val=[x1(:,j);y1(:,j);x(:,j);y(:,j)];
        %x_val=[x_ss;y_ss;x_ss;y_ss];
        %disp('error')
        error_global(:,t)= max(abs(feval(fun,x_val)));
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %generate new path from  x_sim_lev(:,t) for next backward induction 
        x_sim(:,t)=x_sim_lev(:,t);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        for t=t_init:T-1
          x_sim(:,t+1)=x_ss+kx+pp_ss*(x_sim(:,t)-x_ss)+ (1/2)*kron(eye(nx),(x_sim(:,t)-x_ss)')*gg*(x_sim(:,t)-x_ss) +sigm*eps_s(:,t+1);
          y_sim(:,t)  =y_ss+ky+ff_ss*(x_sim(:,t)-x_ss)+ (1/2)*kron(eye(ny),(x_sim(:,t)-x_ss)')*ee*(x_sim(:,t)-x_ss);
        end
        %y_sim(:,T)=ff_ss*x_sim(:,T);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%     
        %y_sim_lev(:,TT)  = ff(:,:,TT-1)*(x_sim_lev(:,TT)-x(:,TT)) +y_ss;    
end
toc
%we still need y at t_end 
%find closest approximation
%%%%%%%%%%%%%%%%%compute error%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
i=1;
pen=norm(xtemp(:,i)-x_sim_lev(:,t_end));
for v=1:j-1
    if norm(x(:,v)-x_sim_lev(:,t_end))<pen
        pen=norm(x(:,v)-x_sim_lev(:,t_end));
        i=v;
    end
end

y_sim_lev(:,t_end)=ff(:,:,i)* (x_sim_lev(:,t_end)-x(:,i))+y(:,i);
%found y at t_end 

%simulate thereafter with ss policies, just for the records
x_sim_lev(:,t_end+1:T)=x_sim(:,t_end+1:T);

%y_sim_lev(:,t_end+1) =ff(:,:,j)* (x_sim(:,t_end+1)-x(:,j))+y(:,j);
y_sim_lev(:,t_end+1:T-1)=y_sim(:,t_end+1:T-1);     
y_sim_lev(:,T)  =ky+ ff_ss*(x_sim(:,T)-x_ss)+...
    (1/2)*kron(eye(ny),(x_sim(:,T)-x_ss)')*ee*(x_sim(:,T)-x_ss) +y_ss;

x_sim_lev(:,1)=x_0;

disp('end')
%compute error f(Ex1,Ey1,x,y) with 2nd order perturbation of the ss.
%rename true policy para as in compute_error
TT=T;
ff=ff_;pp=pp_;x=xt_;y=y_;x1=x1_;y1=y1_;
compute_error_local;
 