%
% Function: solab
%
% Written by Paul Klein with a minor update by Alessandro Mennuni
%
% Format: [f,p, stability] = solab(a,b,nk);
%
% Purpose: Solves for the recursive representation of the stable solution 
% to a system of linear difference equations.
%
% Inputs: Two square matrices a and b and a natural number nk
%
% a and b are the coefficient matrices of the difference equation
%
% a*x(t+1) = b*x(t)
% 
% where x(t) is arranged so that the state variables come first, and
%
% nk is the number of state variables.
%
% Outputs: the decision rule f and the law of motion p. If we write
%
% x(t) = [k(t);u(t)] where k(t) contains precisely the state variables, then
% 
% u(t)   = f*k(t) and
%
% k(t+1) = p*k(t).
%
% Calls: qz, ordqz
%

function [f,p,stability] = solab_stabil(a,b,nk);

[s,t,q,z] = qz(a,b,'real'); %qz(a,b);        % upper triangular factorization of the matrix pencil b-za
[s,t,q,z] = ordqz(s,t,q,z,'udo');   % reordering of generalized eigenvalues with the block inside the unit circle in the upper left
z21 = z(nk+1:end,1:nk);
z11 = z(1:nk,1:nk);

if rank(z11)<nk;
	warning('Invertibility condition violated')
    %error('Invertibility condition violated')
     stability=4;
end

z11i = z11\eye(nk);
s11 = s(1:nk,1:nk);
t11 = t(1:nk,1:nk);

if abs(t(nk,nk))>abs(s(nk,nk)) | abs(t(nk+1,nk+1))<abs(s(nk+1,nk+1));
   warning('Wrong number of stable eigenvalues.');
   disp('In particular:');
else
    stability=1;
end

if abs(t(nk+1,nk+1))<abs(s(nk+1,nk+1));
   disp('Indeterminacy: less eigenvalues  outside the unit circle than jump variables in the system [s1;u1]=A[s;u] where [x;y]=Z[s;u]. See Gomme-Klein page 9.');
   stability=2;
end
if abs(t(nk,nk))>abs(s(nk,nk));
   disp('No stable solution: more eigenvalues outside the unit circle than jump variables in the system [s1;u1]=A[s;u] where [x;y]=Z[s;u].See Gomme-Klein page 9.');
   stability=3;
end

dyn = s11\t11;

f = real(z21*z11i);
p = real(z11*dyn*z11i);